## Créé le 3/7/2020 ##

######################################################
#  ___         __    __  ___         __   ___  ____  #
#  |__| |  |  /  \  /    |__| |  |  /  \  |__| |     #
#  |    |--| |    | ---- |    |--| |    | |\   |--   #
#  |    |  |  \__/  ___/ |    |  |  \__/  | \  |___  #
#                                                    #
######################################################

##
# Met à jour l'affichage graphique des relations (après le chargement d'un projet par exemple)
##
proc Katyusha_Relations_maj {} {
    global relations
    global relations_graphique
    global ID
    foreach {id relation} $relations {
        set ID $id
        set graph [Katyusha_Relations_creer_affichage_graphique $ID $relation]
        # Ajoute la liste temporaire au dictionnaire graphique des relations
        dict set relations_graphique $ID $graph
        puts "Ajout de la relation : [dict get $relation nom]"
        # Créé les lignes reliant la relation aux tables concernées
        Katyusha_Relations_lignes_relation_tables $relation $id
        unset graph id relation
        # Met à jour l'arbre des entités
        maj_arbre_entites
    }
}

proc Katyusha_Relations_MAJ_lignes_relations {} {
    global relations
    foreach {k relation} $relations {
        Katyusha_Relations_lignes_relation_tables $relation $k
    }
}

##
# Créé l'affichage graphique d'une relation
##
proc Katyusha_Relations_creer_affichage_graphique {ID relation} {
    # Récupère le nom de la relation
    set nom [dict get $relation "nom"]
    # Créé le texte d'affichage des attributs
    set texte_liens ""
    set hauteur 20
    # Texte des liens de la relation avec les tables
    foreach {k lien} [dict get $relation "liens"] {
        set table_lien [lindex $lien 0]
        set type_lien [lindex $lien 1]
        set texte_liens "$texte_liens$k : $table_lien -> $type_lien\n"
        set hauteur [expr $hauteur + 20]
    }
    set hauteur [expr $hauteur + 10]
    set texte_attributs "$texte_liens"
    foreach v [dict get $relation "attributs"] {
        set nom_at [dict get $v nom]
        set type_at [dict get $v type]
        set valeur_at [dict get $v valeur]
        set texte_attributs "$texte_attributs$nom_at | $type_at | $valeur_at\n"
        set hauteur [expr $hauteur + 20]
        # Destruction des variables temporaires pour être sûre de ne pas interférer
        # avec les données de la boucle suivante
        unset nom_at type_at valeur_at
    }
    # Calcul la taille de la relation sur le canvas
    set largeur [expr ([Katyusha_Tables_taille_table_graphique $relation] * 8) + 50]
    # Créé l'affichage graphique de la nouvelle relation dans une liste temporaire
    set x [lindex [dict get $relation "coords"] 0]
    set y [lindex [dict get $relation "coords"] 1]
    lappend graph [.mcd.canvas.c create rect [expr $x - [expr $largeur / 2]] [expr $y - 60] [expr $x + [expr $largeur / 2]] [expr $y - 30] -outline #FF7000 -fill #FFE3CD -tag [list relation $ID]]
    lappend graph [.mcd.canvas.c create text [expr $x + 0] [expr $y - 45] -fill black -justify center -text $nom -tag [list relation $ID]]
    lappend graph [.mcd.canvas.c create rect [expr $x - [expr $largeur / 2]] [expr $y - 30] [expr $x + [expr $largeur / 2]] [expr ($y - 30) + $hauteur] -outline #FF7000 -fill #FFE3CD -tag [list relation $ID]]
    lappend graph [.mcd.canvas.c create text [expr $x - [expr ($largeur / 2) - 15]] [expr $y + 5] -fill black -justify left -text $texte_attributs -anchor w -tag [list relation $ID]]
    unset x y hauteur largeur nom relation ID
    return $graph
}

##
# Ajout d'une relation entre plusieurs tables
##
proc Katyusha_ajout_relation {relation_tmp} {
    # Charge les variables nécessaires à l'ajout d'une nouvelle relation
    global relations
    global relations_graphique
    global ID
    # Créé un id pour la nouvelle relation
    set id [expr [dict size $relations]]
    # Récupère le nom de la relation
    set nom [dict get $relation_tmp nom]
    # Si le nom est vide
    if {$nom == ""} {
        set nom "Relation_$id"
        dict set relation_tmp "nom" $nom
        dict unset $relation_tmp "id"
    }
    # Ajoute la nouvelle relation aux relations existantes
    dict set relations $ID $relation_tmp
    # Créé le texte d'affichage des attributs
    set texte_attributs ""
    set hauteur 20
    # Calcul la taille de la relation sur le canvas
    set largeur [expr ([string length $nom] * 8) + 50]
    #set hauteur [expr [dict size $attributs] * 15]
    # Créé l'affichage graphique de la nouvelle relation dans une liste temporaire
    set x [lindex [dict get $relation_tmp "coords"] 0]
    set y [lindex [dict get $relation_tmp "coords"] 1]
    lappend graph [.mcd.canvas.c create rect [expr $x - [expr $largeur / 2]] [expr $y - 60] [expr $x + [expr $largeur / 2]] [expr $y - 30] -outline #FF7000 -fill #FFE3CD -tag [list relation $ID]]
    lappend graph [.mcd.canvas.c create text [expr $x + 0] [expr $y - 45] -fill black -justify center -text $nom -tag [list relation $ID]]
    lappend graph [.mcd.canvas.c create rect [expr $x - [expr $largeur / 2]] [expr $y - 30] [expr $x + [expr $largeur / 2]] [expr ($y - 30) + $hauteur] -outline #FF7000 -fill #FFE3CD -tag [list relation $ID]]
    lappend graph [.mcd.canvas.c create text [expr $x + [expr ($largeur / 2) - 15]] [expr $y - 15] -fill black -justify left -text $texte_attributs -anchor w -tag [list relation $ID]]
    # Trace les lignes pour plus de visibilité
    Katyusha_Relations_lignes_relation_tables $relation_tmp $ID
    # Ajoute la liste temporaire au dictionnaire graphique des tables
    dict set relations_graphique $ID $graph
    puts "Ajout de la relation : [dict get $relation_tmp nom]"
    set ID [expr $ID + 1]
    puts $relations
    unset graph x y relation_tmp nom hauteur largeur texte_attributs
    maj_arbre_entites
}

proc Katyusha_Relations_modification_graphique {id relation} {
    global relations
    global relations_graphique
    
    # Récupère le nom de la relation
    set nom [dict get $relation nom]
    # Créé le texte d'affichage des attributs
    set texte_attributs ""
    set hauteur 20
    foreach v [dict get $relation attributs] {
        set nom_at [dict get $v nom]
        set type_at [dict get $v type]
        set valeur_at [dict get $v valeur]
        set texte_attributs "$texte_attributs$nom_at | $type_at | $valeur_at\n"
        set hauteur [expr $hauteur + 20]
    }
    # Calcul la taille de la table sur le canvas
    set largeur [expr ([Katyusha_Tables_taille_table_graphique $relation] * 8) + 50]
    #set hauteur [expr [dict size [dict get $table_tmp attributs]] * 15]
    # Créé l'affichage graphique de la nouvelle table dans une liste temporaire
    set x [lindex [dict get $relation "coords"] 0]
    set y [lindex [dict get $relation "coords"] 1]
    lappend graph [.mcd.canvas.c create rect [expr $x - [expr $largeur / 2]] [expr $y - 60] [expr $x + [expr $largeur / 2]] [expr $y - 30] -outline #FF7000 -fill #FFE3CD -tag [list relation $id]]
    lappend graph [.mcd.canvas.c create text [expr $x + 0] [expr $y - 45] -fill black -justify center -text $nom -tag [list relation $id]]
    lappend graph [.mcd.canvas.c create rect [expr $x - [expr $largeur / 2]] [expr $y - 30] [expr $x + [expr $largeur / 2]] [expr ($y - 30) + $hauteur] -outline #FF7000 -fill #FFE3CD -tag [list relation $id]]
    lappend graph [.mcd.canvas.c create text [expr $x + [expr ($largeur / 2) - 15]] [expr $y - 15] -fill black -justify left -text $texte_attributs -anchor w -tag [list relation $id]]
    # Ajoute la liste temporaire au dictionnaire graphique des relations
    dict set relations_graphique $id $graph
}

##
# Enregistre les modifications d'une relation
##
proc Katyusha_Relations_modification_relation {id relation} {
    global relations
    suppression_relation $id
    dict set relations $id $relation
    Katyusha_Relations_modification_graphique $id $relation
    maj_arbre_entites
}

##
# Met à jour les coordonnées d'une relation par son ID
##
proc Katyusha_Relations_MAJ_coords {id_relation coords} {
    global relations
    set relation [dict get $relations $id_relation]
    dict set relation "coords" $coords
    dict set relations $id_relation $relation
}

##
# Créé les lignes reliant la relation aux tables qu'elle concerne
##
proc Katyusha_Relations_lignes_relation_tables {relation id_relation} {
    global relations
    global relation_graphiques
    global lignes_graphique
    # Détermine les coordonnées des lignes à tracer
    # Origines des lignes
    set x_origine [lindex [dict get $relation "coords"] 0]
    set y_origine [lindex [dict get $relation "coords"] 1]
    
    set liens [dict get $relation "liens"]
    # Teste si les liens ne sont pas nuls
    if {$liens != ""} {
        foreach {k lien} $liens {
            # Teste si les liens concernent bin une table
            if {[lindex $lien 0] != ""} {
                set nom_table_lien [lindex $lien 0]
                # ID de la table par son nom
                set id_table [Katyusha_Tables_ID_table $nom_table_lien]
                # Récupère les coordonnées graphiques de l'affichage graphique de la table
                set coords_table_lien [Katyusha_Tables_coords_ID $id_table]
                dict set lignes_graphique [dict size $lignes_graphique] [list [.mcd.canvas.c create line $x_origine $y_origine [lindex $coords_table_lien 0] [lindex $coords_table_lien 1] -width 2 -fill #FF7000 -tag [list ligne $id_table]] $id_table $id_relation]
            }
        }
    }
}

##
# Met à jour les coordonnées des lignes reliants la relation aux tables
# La mise à jour des coordonnées ne fonctionnant pas correctement, à chaque fois
# les lignes sont supprimées et remplacées par des lignes correspondants aux nouvelles coordonnées
##
proc Katyusha_Relations_MAJ_ligne_coords {id_relation coords} {
    global lignes_graphique
    
    set x [lindex $coords 0]
    set y [lindex $coords 1]
    
    # Balayage des lignes à la recharche de celles concernants la relation spécifiée
    foreach {k ligne} $lignes_graphique {
        set id_relation_tmp [lindex $ligne 2]
        if {$id_relation_tmp == $id_relation} {
            # Récupère les anciennes coordonnées de la ligne
            set acoords [.mcd.canvas.c coords [lindex $ligne 0]]
            # Créé les nouvelles coordonnées :
            #           [nouveau_x_origine nouveau_y_origine ancien_x_cible ancien_y_cible]
            set ncoords [list $x $y [lindex $acoords 2] [lindex $acoords 3]]
            set id_table [lindex $ligne 1]
            #.mcd.canvas.c coords $ligne $x $y [lindex $acoords 2] [lindex $acoords 3]
            # Créé la nouvelle ligne
            dict set lignes_graphique $k [list [.mcd.canvas.c create line $ncoords -width 2 -fill #FF7000 -tag [list ligne $id_table]] $id_table $id_relation]
            # Et supprimme l'ancienne
            .mcd.canvas.c delete [lindex $ligne 0]
        }
    }
    # Mise à jour de l'affichage graphique
    update
}

##
# Ici, on suppose que les données ont été controlée avant injection
##
proc Katyusha_Relations_ajout_attribut {nom type taille valeur auto pk {graphique 1}} {
    global relation_tmp
    global id_attribut_graphique
    
    set id_attribut_graphique [expr $id_attribut_graphique + 1]
    
    set attributs [dict get $relation_tmp "attributs"]
    puts $attributs
    set id [llength $attributs]
    
    dict set attribut "nom" $nom
    dict set attribut "type" $type
    dict set attribut "taille" $taille
    dict set attribut "valeur" $valeur
    dict set attribut "auto" $auto
    dict set attribut "pk" $pk
    
    lappend attributs $attribut
    
    dict set relation_tmp "attributs" $attributs
    
    if {$graphique == 1} {
        set f ".fen_ajout_relation"
        #
        frame $f.attributs.corps.$id_attribut_graphique
            label $f.attributs.corps.$id_attribut_graphique.nom -text $nom -width 20 -background white -relief solid
            label $f.attributs.corps.$id_attribut_graphique.type -text $type -width 20 -background white -relief solid
            label $f.attributs.corps.$id_attribut_graphique.taille -text $taille -width 20 -background white -relief solid
            label $f.attributs.corps.$id_attribut_graphique.valeur -text $valeur -width 20 -background white -relief solid
            label $f.attributs.corps.$id_attribut_graphique.auto -text $auto -width 20 -background white -relief solid
            label $f.attributs.corps.$id_attribut_graphique.pk -text $pk -width 20 -background white -relief solid
            button $f.attributs.corps.$id_attribut_graphique.edit -text "Éditer" -image $IMG(editer) -command "Katyusha_Relations_modification_attribut $id"
            pack $f.attributs.corps.$id_attribut_graphique.nom $f.attributs.corps.$id_attribut_graphique.type $f.attributs.corps.$id_attribut_graphique.taille $f.attributs.corps.$id_attribut_graphique.valeur $f.attributs.corps.$id_attribut_graphique.auto $f.attributs.corps.$id_attribut_graphique.pk $f.attributs.corps.$id_attribut_graphique.edit -side left
        pack $f.attributs.corps.$id_attribut_graphique -fill x
    }
    puts $relation_tmp
    update
}

proc Katyusha_Relations_controle_relation {relation} {
    set ok 1
    
    return $ok
}

##
# Supprime la relation passée en paramètre
##
proc suppression_relation {relation} {
    global LOCALE
    global relations
    global relations_graphique
    # Récupère le nom de la table
    set nom [dict get [dict get $relations $relation] nom]
    # Supprime la table du tableau général
    dict unset relations $relation
    # Supprime l'affichage de la table
    for {set c 0} {$c < 4} {incr c} {
        .mcd.canvas.c delete [lindex [dict get $relations_graphique $relation] $c]
    }
    dict unset relations_graphique $relation
    maj_arbre_entites
    puts "Relation $nom supprimée"
    unset nom
}

##
# Initialise une relation
##
proc Katyusha_Relations_init_relation {} {
    set relation [dict create]
    dict set relation "attributs" [list]
    dict set relation "liens" [dict create]
    
    return $relation
}

##
# Modifie un attribut
##
proc Katyusha_Relations_modification_attribut {id_attribut} {
    global relation_tmp
    set attributs [dict get $relation_tmp "attributs"]
    puts [lindex $id_attribut]
}
