## Créé le 5/5/2020 ##

######################################################
#  ___         __    __  ___         __   ___  ____  #
#  |__| |  |  /  \  /    |__| |  |  /  \  |__| |     #
#  |    |--| |    | ---- |    |--| |    | |\   |--   #
#  |    |  |  \__/  ___/ |    |  |  \__/  | \  |___  #
#                                                    #
######################################################


##
# Construit le script SQL des tables
# TODO : gestion des clefs étrangères
##
proc Katyusha_tables_sql {tables sgbd} {
    set SQL ""
    # Balayage des tables
    foreach {k table} $tables {
        set nom_table [dict get $table "nom"]
        set attributs_table [dict get $table "attributs"]
        # Si pas de description, ""
        if {[dict exists $table "description"]} {
            set description_table [dict get $table "description"]
        } else {
            set description_table ""
        }
        set attributs_sql ""
        # Balayage des attributs de la table
        foreach attribut $attributs_table {
            set attribut_sql ""
            set nom_attribut [dict get $attribut nom]
            set type_attribut [dict get $attribut type]
            set taille_attribut [dict get $attribut taille]
            set pk_attribut [dict get $attribut pk]
            set auto_attribut [dict get $attribut auto]
            set valeur_attribut [dict get $attribut valeur]
            set null_attribut 1
            # Contruit la taille du type de m'attribut
            if {[lsearch [list "int" "char" "varchar" "long" "double" "float"] $type_attribut] != -1} {
                if {$taille_attribut == 0} {
                    set type_attribut "$type_attribut\(255\)"
                } else {
                    set type_attribut "$type_attribut\($taille_attribut\)"
                }
            }
            set attribut_sql "$attribut_sql$nom_attribut $type_attribut"
            if {$pk_attribut == 1} {
                set attribut_sql "$attribut_sql PRIMARY KEY"
                set null_attribut 0
            }
            if {$auto_attribut == 1} {
                set attribut_sql "$attribut_sql AUTO_INCREMENT"
                set null_attribut 0
            }
            if {$null_attribut == 0} {
                set attribut_sql "$attribut_sql NOT NULL"
            }
            if {$valeur_attribut != ""} {
                set attribut_sql "$attribut_sql DEFAULT '$valeur_attribut'"
            }
            if {$attributs_sql != ""} {
                set attributs_sql "$attributs_sql, \n$attribut_sql"
            } else {
                set attributs_sql "$attributs_sql$attribut_sql"
            }
        }
        # Construit le script de la table
        if {$description_table != "" && $description_table != "\n"} {
            set description_table "\n/*\n$description_table\n*/\n"
        }
        set table "/* Table : $nom_table */\n$description_table\nCREATE TABLE $nom_table (\n$attributs_sql\n) ;\n\n\n"
        # Ajoute le script de la table au script général
        set SQL "$SQL$table"
    }
    return $SQL
}

##
# Renvoie une liste contenant la ou les clefs primaire(s) d'une table selon son nom
##
proc Katyusha_pk_table {nom_table sgbd} {
    global tables
    set attributs [list]
    foreach {k table} $tables {
        set nom_table_liste [dict get $table "nom"]
        # Si le nom correspond
        if {$nom_table_liste == $nom_table} {
            set attributs_table [dict get $table "attributs"]
            # Balayage des attributs à la recherche des clefs primaires
            foreach attribut $attributs_table {
                set pk_attribut [dict get $attribut "pk"]
                # Ajoute le nom de la table (s'il n'est pas déjà inclu) pour une meilleure lisibilité
                # et éviter les doublons
                # Par example :
                #       table.id devient table.id_table
                set nom_attribut [dict get $attribut "nom"]
                dict set attribut "nom_origine" $nom_attribut
                # Supprime les incrémentations automatiques
                dict set attribut "auto" 0
                dict set attribut "valeur" ""
                if {[string first $nom_table $nom_attribut] == -1} {
                    dict set attribut "nom" "$nom_attribut\_$nom_table"
                }
                if {$pk_attribut == 1} {
                    lappend attributs $attribut
                }
            }
        }
    }
    return $attributs
}

##
#
##
proc Katyusha_ajout_fk_table {table_lien table_lier pk_table_lier sgbd} {
    set SQL ""
    foreach pk $pk_table_lier {
        set nom_attribut [dict get $pk "nom"]
        set nom_origine_attribut [dict get $pk "nom_origine"]
        set type_attribut [dict get $pk "type"]
        set SQL "$SQL\nALTER TABLE $table_lien ADD COLUMN $nom_attribut $type_attribut NOT NULL ;"
        set SQL "$SQL\nALTER TABLE $table_lien ADD CONSTRAINT FK_$table_lier\_$nom_attribut FOREIGN KEY \($nom_attribut\) REFERENCES $table_lier\($nom_origine_attribut\) ;\n"
    }
    return $SQL
}

##
# Détermine si une nouvelle table doit être créer
#       1 pour une nouvelle table
#       0 sinon
##
proc Katyusha_GenerationSQL_liens_n_table {liens} {
    set n_table 1
    #set tmp 0
    
    if {[dict size $liens] == 2} {
        foreach {k lien} $liens {
            set table_lien [lindex $lien 0]
            set n_lien [lindex $lien 1]
            if {$n_table == 1} {
                if {$n_lien == "1.1"} {
                    set n_table 0
                    #set tmp 1
                } else {
                }
            }
        }
    }
    
    #set n_table $tmp
    
    return $n_table
}

proc Katyusha_GenerationSQL_attributs_n_table {liens sgbd} {
    set SQL ""
    foreach {k lien} $liens {
        set table_lien [lindex $lien 0]
        set n_lien [lindex $lien 1]
        if {$n_lien == "0.n" || $n_lien == "0.1" || $n_lien == "1.n"} {
            # Liste des clefs primaires de la table
            set liste_pk_table [Katyusha_pk_table $table_lien $sgbd]
            # Balayage des clefs primaires
            foreach attribut $liste_pk_table {
                lappend attributs $attribut
            }
        }
    }
    return $attributs
}

##
# Retourne le code SQL pour une relation
##
proc Katyusha_GenerationSQL_relation {relation sgbd} {
    set SQL ""
    set nom_relation [dict get $relation "nom"]
    # Au cas ou il faudrait créer une table spécifique à cette relation
    set attributs [dict create]
    set c 0
    # Balayage des liens
    set liens [dict get $relation "liens"]
    
    # Détermine si une nouvelle table doit être créée
    set n_table [Katyusha_GenerationSQL_liens_n_table $liens]
    
    if {$n_table == 1} {
        # Créé les attributs de la nouvelle table
        dict set table_tmp "nom" $nom_relation
        dict set table_tmp "description" ""
        dict set table_tmp "attributs" [Katyusha_GenerationSQL_attributs_n_table $liens $sgbd]
    } else {
        set table_tmp "null"
    }
    
    return [list $SQL $table_tmp]
}

proc Katyusha_GenerationSQL_relations {relations sgbd} {
    set SQL ""
    set tables_tmp [list]
    
    foreach {k relation} $relations {
        set sql_table [lindex [Katyusha_GenerationSQL_relation $relation $sgbd] 0]
        set table [lindex [Katyusha_GenerationSQL_relation $relation $sgbd] 1]
        if {$table != "null"} {
            lappend tables_tmp $table
        } else {
            set liens [dict get $relation "liens"]
            set lien1 [dict get $liens 0]
            set lien2 [dict get $liens 1]
            set table_lien [lindex $lien1 0]
            set table_liee [lindex $lien2 0]
            set pk_table_liee [Katyusha_pk_table $table_liee $sgbd]
            #Katyusha_ajout_fk_table $table_lien $table_liee $pk_table_liee $sgbd
        }
        set SQL "$SQL$sql_table"
    }
    
    return [list $SQL $tables_tmp]
}

##
# Ajoute un drop et create au script SQL
##
proc Katyusha_GenerationSQL_base {sgbd} {
    global CONFIGS
    global MCD
    
    set SQL ""
    
    # Si drop
    if {$MCD(drop) == 1} {
        set SQL "$SQL[Katyusha_SQL_drop_base $MCD(nom) $sgbd]"
    }
    
    return $SQL
}

proc Katyusha_GenerationSQL_liens_egaux {liens} {
    set res 0
    set lien1 "null"
    
    foreach {k lien} $liens {
        set lien2 [lindex $lien 1]
        if {$lien1 == $lien2} {
            set res 1
        } else {
            set res 0
        }
        set lien1 $lien2
    }
    
    return $res
}

##
# Créé le code SQL des clefs étrangères à ajouter
##
proc Katyusha_GenerationSQL_FK {tables relations sgbd} {
    set SQL ""
    foreach {k relation} $relations {
        set nom_relation [dict get $relation "nom"]
        # Récupère et analyse les liens de la relation
        set liens [dict get $relation "liens"]
        set n_table [Katyusha_GenerationSQL_liens_n_table $liens]
        
        if {$n_table == 1} {
        foreach {k lien} $liens {
            set table_lien [lindex $lien 0]
            set n_lien [lindex $lien 1]
            set pk_table_lien [Katyusha_pk_table $table_lien $sgbd]
                foreach pk $pk_table_lien {
                    set nom_pk [dict get $pk "nom"]
                    set nom_origine_pk [dict get $pk "nom_origine"]
                    set SQL "$SQL[Katyusha_SQL_ajout_fk $nom_relation $table_lien $nom_pk $nom_origine_pk]" 
                }
        }
        } else {
            set liens [dict get $relation "liens"]
            set lien1 [dict get $liens 0]
            set lien2 [dict get $liens 1]
            if {[Katyusha_GenerationSQL_liens_egaux $liens] == 1} {
                set table_lien [lindex $lien1 0]
                set table_liee [lindex $lien2 0]
            } else {
                set type_lien1 [lindex $lien1 1]
                if {$type_lien1 == "1.1"} {
                    set table_lien [lindex $lien2 0]
                    set table_liee [lindex $lien1 0]
                } else {
                    set table_lien [lindex $lien1 0]
                    set table_liee [lindex $lien2 0]
                }
            }
            set pk_table_liee [Katyusha_pk_table $table_liee $sgbd]
            foreach pk $pk_table_liee {
                set nom_pk [dict get $pk "nom"]
                set nom_origine_pk [dict get $pk "nom_origine"]
                set type_pk [dict get $pk "type"]
                set SQL "$SQL[Katyusha_SQL_ajout_colone $table_lien $nom_pk $type_pk]"
                set SQL "$SQL[Katyusha_SQL_ajout_fk $table_lien $table_liee $nom_pk $nom_origine_pk]"
            }
        }
    }
    return $SQL
}

proc Katyusha_SQL_FK2 {} {
    global fk
}

##
# Génère le script SQL du MCD
##
proc Katyusha_generation_sql {sgbd} {
    global tables
    set tables_tmp $tables
    global relations
    set relations_tmp $relations
    global id_projet
    global rep_mcd
    global version
    global fk
    global MCD
    
    set SQL "/*\nScript généré automatiquement par Katyusha MCD v$version\n*/\n\n"
    
    set sql_base [Katyusha_GenerationSQL_base $sgbd]
    set SQL "$SQL$sql_base"
    
    set retour_relations [Katyusha_GenerationSQL_relations $relations_tmp $sgbd]
    set relations_sql [lindex $retour_relations 0]
    # Ajoute les nouvelles tables créées pour les relations
    foreach table_tmp [lindex $retour_relations 1] {
        set c [dict size $tables_tmp]
        dict set tables_tmp $c $table_tmp
    }
    set tables_sql [Katyusha_tables_sql $tables_tmp $sgbd]
    set SQL "$SQL$tables_sql$relations_sql"
    set SQL "$SQL[Katyusha_GenerationSQL_FK $tables $relations $sgbd]"
    # Défini le chemin d'enregistrement du fichier SQL
    if {$id_projet == -1} {
        set id_projet [llength [glob -nocomplain -dir $rep_mcd "projet*"]]
        file mkdir "$rep_mcd/projet$id_projet"
    }
    set id_fichier [llength [glob -nocomplain -dir "$rep_mcd/projet$id_projet" "$MCD(nom)*.sql"]]
    set fichier "$rep_mcd/projet$id_projet/$MCD(nom)$id_fichier\-$sgbd.sql"
    # Enregistre dans le fichier
    set stream [open $fichier "w+"]
    puts $stream $SQL
    close $stream
    return [list $SQL $fichier]
}

##
# Vérifie le MCD avant génération du script SQL
##
proc Katyusha_verification_mcd_sql {sgbd} {
    set err [Katyusha_verification_mcd]
    set res_sql [list 0 0]
    if {$err == 1} {
        # Informe qu'il y a un problème
        set res_info [tk_messageBox -icon warning -type ok -message "Il y a une erreur dans le MCD qui empêche la génération du script SQL pour $sgbd"]
    } else {
        set res_sql [Katyusha_generation_sql $sgbd]
    }
    return $res_sql
}
