## Créé le 3/7/2020 ##

######################################################
#  ___         __    __  ___         __   ___  ____  #
#  |__| |  |  /  \  /    |__| |  |  /  \  |__| |     #
#  |    |--| |    | ---- |    |--| |    | |\   |--   #
#  |    |  |  \__/  ___/ |    |  |  \__/  | \  |___  #
#                                                    #
######################################################


##
#
##
proc maj_tables {} {
    global tables
    global tables_graphique
    global ID
    foreach {id table} $tables {
        set ID $id
        set graph [Katyusha_Tables_creer_affichage_graphique $ID $table]
        # Ajoute la liste temporaire au dictionnaire graphique des tables
        dict set tables_graphique $ID $graph
        puts "Ajout de la table : [dict get $table nom]"
        unset graph id table
        # Met à jour l'arbre des entités
        maj_arbre_entites
    }
}

##
# Créé l'affichage graphique d'une table
##
proc Katyusha_Tables_creer_affichage_graphique {ID table} {
    # Récupère le nom de la table
    set nom [dict get $table "nom"]
    # Créé le texte d'affichage des attributs
    set texte_attributs ""
    set hauteur 20
    foreach v [dict get $table attributs] {
        set nom_at [dict get $v nom]
        set type_at [dict get $v type]
        set valeur_at [dict get $v valeur]
        set pk_at [dict get $v pk]
        if {$pk_at == 1} {
            set pk_at " - PK"
        } else {
            set pk_at ""
        }
        set texte_attributs "$texte_attributs$nom_at | $type_at | $valeur_at$pk_at\n"
        set hauteur [expr $hauteur + 20]
        # Destruction des variables temporaires pour être sûre de ne pas interférer
        # avec les données de la boucle suivante
        unset nom_at type_at valeur_at
    }
    # Calcul la taille de la table sur le canvas
    set largeur [expr ([Katyusha_Tables_taille_table_graphique $table] * 8) + 50]
    # Créé l'affichage graphique de la nouvelle table dans une liste temporaire
    set x [lindex [dict get $table "coords"] 0]
    set y [lindex [dict get $table "coords"] 1]
    lappend graph [.mcd.canvas.c create rect [expr $x - [expr $largeur / 2]] [expr $y - 80] [expr $x + [expr $largeur / 2]] [expr $y - 40] -outline #FF7000 -fill #FFB57B -tag [list table $ID]]
    lappend graph [.mcd.canvas.c create text [expr $x + 0] [expr $y - 60] -fill black -justify center -text $nom -tag [list table $ID]]
    lappend graph [.mcd.canvas.c create rect [expr $x - [expr $largeur / 2]] [expr $y - 40] [expr $x + [expr $largeur / 2]] [expr ($y - 40) + $hauteur] -outline #FF7000 -fill #FFFFFF -tag [list table $ID]]
    lappend graph [.mcd.canvas.c create text [expr $x - [expr ($largeur / 2) - 15]] [expr $y - 10] -fill black -justify left -text $texte_attributs -anchor w -tag [list table $ID]]
    unset x y hauteur largeur nom table ID
    return $graph
}

##
# Ajoute une table
##
proc ajout_table {table_tmp} {
    # Charge la variable globale contenant toutes les tables
    global tables
    global tables_graphique
    global ID
    # Créé un id pour la nouvelle table
    set id [expr [dict size $tables]]
    # Ajoute la nouvelle table aux tables existantes
    dict set tables $ID $table_tmp
    set graph [Katyusha_Tables_creer_affichage_graphique $ID $table_tmp]
    # Ajoute la liste temporaire au dictionnaire graphique des tables
    dict set tables_graphique $ID $graph
    puts "Ajout de la table : [dict get $table_tmp nom]"
    set ID [expr $ID + 1]
    unset graph id
    # Met à jour l'arbre des entités
    maj_arbre_entites
}

proc Katyusha_Tables_modification_graphique {id table} {
    global tables
    global tables_graphique
    
    set graph [Katyusha_Tables_creer_affichage_graphique $id $table]
    # Ajoute la liste temporaire au dictionnaire graphique des tables
    dict set tables_graphique $id $graph
}

##
# Enregistre les modifications d'une table
##
proc Katyusha_Tables_modification_table {id table} {
    global tables
    suppression_table $id
    dict set tables $id $table
    Katyusha_Tables_modification_graphique $id $table
    maj_arbre_entites
}

##
# Supprime la table passée en paramètre
##
proc suppression_table {{table "null"}} {
    global LOCALE
    global tables
    global tables_graphique
    global id_entite
    
    if {$table == "null"} {
        set table $id_entite
    }
    # Récupère le nom de la table
    set nom [dict get [dict get $tables $table] nom]
    # Supprime la table du tableau général
    dict unset tables $table
    # Supprime l'affichage de la table
    for {set c 0} {$c < 4} {incr c} {
        .mcd.canvas.c delete [lindex [dict get $tables_graphique $table] $c]
    }
    dict unset tables_graphique $table
    maj_arbre_entites
    puts "Table $nom supprimée"
    unset nom
}

##
# Détermnie la taille en nombre de caractères d'une table pour l'affichage graphique
##
proc Katyusha_Tables_taille_table_graphique {table} {
    set taille 0
    set taille [string length [dict get $table "nom"]]
    set attributs [dict get $table "attributs"]
    foreach attribut $attributs {
        set taille_tmp [string length "[dict get $attribut nom] | [dict get $attribut type]"]
        if {$taille_tmp > $taille} {
            set taille $taille_tmp
        }
    }
    return $taille
}

##
# Trouve l'ID d'une table par son nom
##
proc Katyusha_Tables_ID_table {nom_table} {
    global tables
    
    set id -1
    foreach {k table} $tables {
        set nom_table_tmp [dict get $table "nom"]
        if {$nom_table_tmp == $nom_table} {
            set id $k
        }
    }
    return $id
}

##
# Retourne les coordonnées de l'affichage graphique d'une table par son ID
##
proc Katyusha_Tables_coords_ID {id_table} {
    global tables
    set table [dict get $tables $id_table]
    set coords [dict get $table "coords"]
    return $coords
}


##
# Met à jour les coordonnées d'une table par son ID
##
proc Katyusha_Tables_MAJ_coords {id_table coords} {
    global tables
    set table [dict get $tables $id_table]
    dict set table "coords" $coords
    dict set tables $id_table $table
}

##
#
##
proc Katyusha_Tables_MAJ_ligne_coords {id_table coords} {
    global lignes_graphique
    
    set x [lindex $coords 0]
    set y [lindex $coords 1]
    
    # Balayage des lignes à la recherche de celles concernants la table spécifiée
    foreach {k ligne} $lignes_graphique {
        set id_table_tmp [lindex $ligne 1]
        if {$id_table_tmp == $id_table} {
            # Récupère les anciennes coordonnées de la ligne
            set acoords [.mcd.canvas.c coords [lindex $ligne 0]]
            # Créé les nouvelles coordonnées :
            #           [nouveau_x_origine nouveau_y_origine ancien_x_cible ancien_y_cible]
            set ncoords [list [lindex $acoords 0] [lindex $acoords 1] $x $y]
            set id_relation [lindex $ligne 2]
            #.mcd.canvas.c coords $ligne $x $y [lindex $acoords 2] [lindex $acoords 3]
            # Créé la nouvelle ligne
            dict set lignes_graphique $k [list [.mcd.canvas.c create line $ncoords -width 2 -fill #FF7000 -tag [list ligne $id_table]] $id_table $id_relation]
            # Et supprimme l'ancienne
            .mcd.canvas.c delete [lindex $ligne 0]
        }
    }
    # Mise à jour de l'affichage graphique
    update
}

proc Katyusha_Tables_controle_table {table} {
    set ok 1
    
    return $ok
}

##
#
##
proc Katyusha_Tables_controle_attribut {nom type taille valeur auto pk} {
    set ok 1
    # Contrôle avant acceptation
    if {$nom == ""} {
        set ok 0
    }
    if {[lsearch -exact [Katyusha_SQL_liste_types] $type] < 0} {
        set ok 0
    }
    if {$taille < 0 || $taille > 255} {
        set ok 0
    }
    if {$valeur == ""} {
        set ok 0
    }
    if {$auto != 0 && $auto != 1} {
        set ok 0
    }
    if {$pk != 0 && $pk != 1} {
        set ok 0
    }
    
    return $ok
}

##
# Ici, on suppose que les données ont été controlée avant injection
##
proc Katyusha_Tables_ajout_attribut {nom type taille valeur auto pk {graphique 1}} {
    global table_tmp
    global id_attribut_graphique
    global IMG
    
    set id_attribut_graphique [expr $id_attribut_graphique + 1]
    
    set attributs [dict get $table_tmp "attributs"]
    set id [llength $attributs]
    
    dict set attribut "nom" $nom
    dict set attribut "type" $type
    dict set attribut "taille" $taille
    dict set attribut "valeur" $valeur
    dict set attribut "auto" $auto
    dict set attribut "pk" $pk
    
    lappend attributs $attribut
    
    dict set table_tmp "attributs" $attributs
    
    if {$graphique == 1} {
        set f ".fen_ajout_table"
        #
        frame $f.attributs.corps.$id_attribut_graphique
            label $f.attributs.corps.$id_attribut_graphique.nom -text $nom -width 20 -height 2 -background white -relief solid
            label $f.attributs.corps.$id_attribut_graphique.type -text $type -width 20 -height 2 -background white -relief solid
            label $f.attributs.corps.$id_attribut_graphique.taille -text $taille -width 20 -height 2 -background white -relief solid
            label $f.attributs.corps.$id_attribut_graphique.valeur -text $valeur -width 20 -height 2 -background white -relief solid
            label $f.attributs.corps.$id_attribut_graphique.auto -text $auto -width 20 -height 2 -background white -relief solid
            label $f.attributs.corps.$id_attribut_graphique.pk -text $pk -width 20 -height 2 -background white -relief solid
            button $f.attributs.corps.$id_attribut_graphique.edit -text "Éditer" -image $IMG(editer) -command "Katyusha_Tables_modification_attribut $id"
            pack $f.attributs.corps.$id_attribut_graphique.nom $f.attributs.corps.$id_attribut_graphique.type $f.attributs.corps.$id_attribut_graphique.taille $f.attributs.corps.$id_attribut_graphique.valeur $f.attributs.corps.$id_attribut_graphique.auto $f.attributs.corps.$id_attribut_graphique.pk $f.attributs.corps.$id_attribut_graphique.edit -side left
        pack $f.attributs.corps.$id_attribut_graphique -fill x
    }
    puts $table_tmp
}

##
#
##
proc Katyusha_Tables_suppression_attribut_table {table nom_attribut} {
    set attributs [dict get $table "attributs"]
    set n_attributs [list]
    foreach attribut $attributs {
        if {[dict get $attribut "nom"] != $nom_attribut} {
            lappend n_attributs $attribut
        }
    }
    dict set table "attributs" $n_attributs
    
    return $table
}

##
# Initialise une table
##
proc Katyusha_Tables_init_table {} {
    set table [dict create]
    dict set table "attributs" [list]
    
    return $table
}

##
# Modifie un attribut
##
proc Katyusha_Tables_modification_attribut {id_attribut} {
    global table_tmp
    set attributs [dict get $table_tmp "attributs"]
    puts [lindex $id_attribut]
}
